using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using XNAGeometryObject;

namespace XNATextureAddressMode
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        // quad
        Quad[] quad = new Quad[4];
        Texture2D[] texture = new Texture2D[2];
        // texture address mode SamplerState
        SamplerState[] samplerState = new SamplerState[4];
    
        // effect
        BasicEffect effect;
        Matrix projection;
        Matrix view;
        Matrix world;

        // input
        GamePadState currentGamePadState;
        GamePadState prevGamePadState;
        KeyboardState currentKeyboardState;
        KeyboardState prevKeyboardState;
        float angle = 0.0f;
        float zoom = 1.0f;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            quad[0] = new Quad(new Vector3(-1.1f, 1.1f, 0.0f), Vector3.Backward, Vector3.Up, 2, 2, new Vector2(0.0f, 0.0f), new Vector2(3.0f, 0.0f), new Vector2(0.0f, 3.0f), new Vector2(3.0f, 3.0f));
            quad[1] = new Quad(new Vector3(1.1f, 1.1f, 0.0f), Vector3.Backward, Vector3.Up, 2, 2, new Vector2(0.0f, 0.0f), new Vector2(3.0f, 0.0f), new Vector2(0.0f, 3.0f), new Vector2(3.0f, 3.0f));
            quad[2] = new Quad(new Vector3(-1.1f, -1.1f, 0.0f), Vector3.Backward, Vector3.Up, 2, 2, new Vector2(0.0f, 0.0f), new Vector2(3.0f, 0.0f), new Vector2(0.0f, 3.0f), new Vector2(3.0f, 3.0f));
            quad[3] = new Quad(new Vector3(1.1f, -1.1f, 0.0f), Vector3.Backward, Vector3.Up, 2, 2, new Vector2(0.0f, 0.0f), new Vector2(3.0f, 0.0f), new Vector2(0.0f, 3.0f), new Vector2(3.0f, 3.0f));

            samplerState[0] = new SamplerState() { Filter = TextureFilter.Linear, AddressU = TextureAddressMode.Clamp, AddressV = TextureAddressMode.Clamp };
            samplerState[1] = new SamplerState() { Filter = TextureFilter.Linear, AddressU = TextureAddressMode.Wrap, AddressV = TextureAddressMode.Wrap };
            samplerState[2] = new SamplerState() { Filter = TextureFilter.Linear, AddressU = TextureAddressMode.Mirror, AddressV = TextureAddressMode.Mirror };
            samplerState[3] = new SamplerState() { Filter = TextureFilter.Linear, AddressU = TextureAddressMode.Wrap, AddressV = TextureAddressMode.Mirror };

            InitializeTransform();
            InitializeEffect();

            base.Initialize();
        }

        /// <summary>
        /// Initializes the transforms used by the game.
        /// </summary>
        private void InitializeTransform()
        {
            // world matrix
            world = Matrix.Identity;

            // view matrix
            view = Matrix.CreateLookAt(new Vector3(0, 0, 7.0f), Vector3.Zero, Vector3.Up);

            // projection matrix
            float aspectRatio = (float)GraphicsDevice.Viewport.Width / (float)GraphicsDevice.Viewport.Height;
            projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4, aspectRatio, 1.0f, 1000.0f);
        }

        /// <summary>
        /// Initializes the effect (loading, parameter setting, and technique selection)
        /// used by the game.
        /// </summary>
        private void InitializeEffect()
        {
            // Initialize BasicEffect
            effect = new BasicEffect(GraphicsDevice);
            effect.Alpha = 1.0f;
            effect.PreferPerPixelLighting = true;

            effect.DiffuseColor = new Vector3(1.0f, 1.0f, 1.0f);    // object material
            effect.SpecularColor = new Vector3(1.0f, 1.0f, 1.0f);
            effect.SpecularPower = 12.0f;
            effect.AmbientLightColor = new Vector3(0.75f, 0.75f, 0.75f);

            effect.DirectionalLight0.Enabled = true;
            effect.DirectionalLight0.DiffuseColor = Vector3.One;
            effect.DirectionalLight0.Direction = Vector3.Normalize(new Vector3(1.0f, 1.0f, 0.0f));
            effect.DirectionalLight0.SpecularColor = Vector3.One;

            effect.DirectionalLight1.Enabled = true;
            effect.DirectionalLight1.DiffuseColor = new Vector3(1.0f, 1.0f, 1.0f);
            effect.DirectionalLight1.Direction = Vector3.Normalize(new Vector3(-1.0f, -1.0f, 1.0f));
            effect.DirectionalLight1.SpecularColor = new Vector3(0.5f, 0.5f, 0.5f);

            effect.LightingEnabled = true;
            effect.TextureEnabled = true;

            effect.World = world;
            effect.View = view;
            effect.Projection = projection;
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
            texture[0] = Content.Load<Texture2D>("xna_logo");
            texture[1] = Content.Load<Texture2D>("crate");
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here
            HandleInput();

            base.Update(gameTime);
        }

        /// <summary>
        /// Determines which primitive to draw based on input from the keyboard
        /// or game pad.
        /// </summary>
        protected void HandleInput()
        {
            CheckGamePadInput();
            CheckKeyboardInput();
        }

        private void CheckGamePadInput()
        {
            prevGamePadState = currentGamePadState;
            currentGamePadState = GamePad.GetState(PlayerIndex.One);
        }

        private void CheckKeyboardInput()
        {
            prevKeyboardState = currentKeyboardState;
            currentKeyboardState = Keyboard.GetState(PlayerIndex.One);

            if (currentKeyboardState.IsKeyDown(Keys.Escape))
            {
                this.Exit();
            }
            else if (currentKeyboardState.IsKeyDown(Keys.X)
                && !prevKeyboardState.IsKeyDown(Keys.X))
            {
                angle += 5.0f;
            }
            else if (currentKeyboardState.IsKeyDown(Keys.Y)
                && !prevKeyboardState.IsKeyDown(Keys.Y))
            {
                zoom -= 0.5f;
            }
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here
            world = Matrix.CreateFromYawPitchRoll(0.0f, 0.0f, MathHelper.ToRadians(angle)) * Matrix.CreateTranslation(0, 0, zoom);
            effect.World = world;

            // effect
            foreach (EffectPass pass in effect.CurrentTechnique.Passes)
            {
                pass.Apply();
                // Set texture filter and address mode
                GraphicsDevice.SamplerStates[0] = samplerState[0];
                effect.Texture = texture[0];
                // Draw the quad
                GraphicsDevice.DrawUserIndexedPrimitives<VertexPositionNormalTexture>(PrimitiveType.TriangleList,
                    quad[0].Vertices, 0, 4, quad[0].Indices, 0, 2);

                pass.Apply();
                // Set texture filter and address mode
                GraphicsDevice.SamplerStates[0] = samplerState[1];
                effect.Texture = texture[1];
                //effect.TextureEnabled = true;
                // Draw the quad
                GraphicsDevice.DrawUserIndexedPrimitives<VertexPositionNormalTexture>(PrimitiveType.TriangleList,
                    quad[1].Vertices, 0, 4, quad[1].Indices, 0, 2);

                pass.Apply();
                // Set texture filter and address mode
                GraphicsDevice.SamplerStates[0] = samplerState[2];
                effect.Texture = texture[0];
                //effect.TextureEnabled = true;
                // Draw the quad
                GraphicsDevice.DrawUserIndexedPrimitives<VertexPositionNormalTexture>(PrimitiveType.TriangleList,
                    quad[2].Vertices, 0, 4, quad[2].Indices, 0, 2);

                pass.Apply();
                // Set texture filter and address mode
                GraphicsDevice.SamplerStates[0] = samplerState[3];
                effect.Texture = texture[1];
                //effect.TextureEnabled = true;
                // Draw the quad
                GraphicsDevice.DrawUserIndexedPrimitives<VertexPositionNormalTexture>(PrimitiveType.TriangleList,
                    quad[3].Vertices, 0, 4, quad[3].Indices, 0, 2);
            }

            base.Draw(gameTime);
        }
    }
}
